-- Get the addon name and the addon object
local swmName, swm = ...
local swmNameFormatted = "|cFF88FF44" .. swmName .. "|r"

--------------------------------------
--									--
--			ALERTS					--
--									--
--------------------------------------

-- Alert the player to click the pet macro button and summon the active pet.
local okToAlert = true

-- Just a simple semaphore to delay between reminders to summon a missing pet.
function SetOkToAlert()
	okToAlert = true
end

-- This warns the player to summon their pet. Only used during combat.
function AlertSummon()
	if okToAlert then
		okToAlert = false
		if svShowAlertText then
			AlertMessage("SUMMON YOUR PET!!")
		end
		if svPlayAlertSound then PlaySoundFile("interface\\addons\\staywithme\\assets\\bark.mp3", "SFX") end
		C_Timer.After(6, SetOkToAlert)
	end
end

-- Create text frame for alert messages.
local alertFrame = CreateFrame("Frame", nil, UIParent)
local alertFrameTime
local alertMessageShowing = false

-- Set up the alert message frame.
function AlertFrameInitialize()
	alertFrame:ClearAllPoints()
	alertFrame:SetHeight(300)
	alertFrame:SetWidth(500)
	alertFrame:Hide()
	alertFrame.text = alertFrame:CreateFontString(nil, "BACKGROUND", "SystemFont_OutlineThick_WTF")
	alertFrame.text:SetAllPoints()
	alertFrame:SetPoint("CENTER", 0, 200)
end

-- Set up the handler for scaling and fading alert messages.
alertFrame:SetScript("OnUpdate", function(self, ...)
	if alertMessageShowing then
		if alertFrameTime < GetTime() - 1.5 then
			local newAlpha = max(self:GetAlpha() - .05, 0)
			self:SetAlpha(newAlpha)
			local newScale = max(self:GetScale() - .025, 1)
			self:SetScale(newScale)
			if newAlpha == 0 then
				self:Hide()
				alertMessageShowing = false
			end
		end
	end
end)

-- Displays a message. Used both for status messages and pet summon warning.
function AlertMessage(message)
	alertFrameTime = GetTime()
	alertFrame.text:SetText("|cFFFFFFFF" .. message .. "|r")
	alertFrame:SetAlpha(1)
	alertFrame:SetScale(1.5)
	alertFrame:Show()
	alertMessageShowing = true
end

--------------------------------------
--									--
--			MACRO BUTTON			--
--									--
--------------------------------------

-- Create draggable macro button for summon/resummon.
local petButton = CreateFrame("Button", "swmMacro", UIParent, "SecureActionButtonTemplate")
petButton:SetAttribute("type1", "macro")
petButton:SetAttribute("macrotext", [[/script print('StayWithMe: Not initialized yet.')]])
petButton:SetWidth(64)
petButton:SetHeight(64)
petButton:SetClampedToScreen(true)
petButton:RegisterEvent("ADDON_LOADED")				-- So we can make sure saved vars loaded.
petButton:RegisterEvent("PLAYER_ENTERING_WORLD")	-- When entering new zone or subzone.
petButton:RegisterEvent("PLAYER_REGEN_DISABLED")	-- When entering combat.

-- Set up 'empty' texture and default the button to that.
local txEmpty = petButton:CreateTexture(nil,"BACKGROUND")
txEmpty:SetTexture("Interface\\ICONS\\INV_Box_PetCarrier_01.blp")
txEmpty:SetAllPoints(petButton)

-- Set up pet icon texture.
local txPet = petButton:CreateTexture(nil, "BACKGROUND")
txPet:SetAllPoints(petButton)

-- Set up button highlight texture.
local txHilight = petButton:CreateTexture(nil, "HIGHLIGHT")
txHilight:SetTexture("Interface\\BUTTONS\\ButtonHilight-Square.blp")
txHilight:SetAllPoints(petButton)
petButton:SetHighlightTexture(txHilight)

-- Set button icon based on the saved variable svActivePet
local petDisplayName

-- Set the button's summoning macro and icon based on the active pet. The macro is constructed to be
--	able to handle both combat and non-combat situations. It will also notify you that it's summoning
--	and which pet is being summoned. Note that this macro is attached directly to the button and
--	doesn't interfere with normal macros.
function UpdatePetButtonIcon(verbose)
	if svActivePet == nil then
		petButton:SetNormalTexture(txEmpty)
		petButton:SetAttribute("macrotext", "/script ToggleCollectionsJournal(2)")
	else
		local _,petCustomName,_,_,_,_,_,petName,petIcon = C_PetJournal.GetPetInfoByPetID(svActivePet)
				
		if petIcon == nil then
			print("|cFFFF2200" .. swmNameFormatted .. ": Nil petIcon!!|r for " .. svActivePet .. ". Please report this bug.")
		else
			txPet:SetTexture(petIcon)
			petButton:SetNormalTexture(txPet)

			if petCustomName == nil then
				petDisplayName = petName
			else
				petDisplayName = petCustomName
			end
			-- local mtext = "/summonpet [combat] " .. petDisplayName .."\n/run AlertMessage('".. petDisplayName .. " summoned.');\n/stopmacro [combat]\n/run p = C_PetJournal.GetSummonedPetGUID();if p ~= '" .. svActivePet .. "' then C_PetJournal.SummonPetByGUID('" ..  svActivePet .. "') end;AlertMessage('".. petDisplayName .. " summoned.');"
			local mtext = "/summonpet [combat] " .. petDisplayName .."\n/run AlertMessage(\"".. petDisplayName .. " summoned.\");\n/stopmacro [combat]\n/run p = C_PetJournal.GetSummonedPetGUID();if p ~= \"" .. svActivePet .. "\" then C_PetJournal.SummonPetByGUID(\"" ..  svActivePet .. "\") end;AlertMessage(\"".. petDisplayName .. " summoned.\");"
			if verbose then
				print(swmNameFormatted .. ": Macro set to |cFF44FFFF" .. mtext .. "|r")
			end
			petButton:SetAttribute("macrotext", mtext)
		end
	end
end

-- Right mouse button clears the current pet from the macro button. With modifiers, it toggles
--	settings for alerts.
petButton:SetScript("OnMouseUp", function(self, button)
	if button == "RightButton" then
		if petButton:IsMouseOver() then
			if IsShiftKeyDown() then
				PlaySoundFile("Sound/Interface/iUiInterfaceButtonA.ogg")
				svShowAlertText = not svShowAlertText
				if svShowAlertText then
					AlertMessage("Alert text is on.")
				else
					AlertMessage("Alert text is off.")
				end
			elseif IsControlKeyDown() then
				PlaySoundFile("Sound/Interface/iUiInterfaceButtonA.ogg")
				svPlayAlertSound = not svPlayAlertSound
				if svPlayAlertSound then
					AlertMessage("Alert sound is on.")
				else
					AlertMessage("Alert sound is off.")
				end
			else
				-- If we're not in combat we can dismiss the pet as well as clearing the button.
				UnsummonActivePet()
				-- Clear our active pet saved variable and update the button image.
				svActivePet = nil
				UpdatePetButtonIcon(false)
			end
		end
	end
end)

-- Make the button draggable.
petButton:SetMovable(true)
petButton:EnableMouse(true)
petButton:RegisterForDrag("LeftButton")
petButton:SetScript("OnDragStart", petButton.StartMoving)

petButton:SetScript("OnDragStop", function(self, event, ...)
	self:StopMovingOrSizing()
	svButtonLoc.hpos = self:GetLeft()
	svButtonLoc.vpos = self:GetBottom()
	UpdatePetButtonIcon(false)
end)

-- If a pet is dropped on the button then set the button to summon that pet.
petButton:SetScript("OnReceiveDrag", function(self, event, ...)
	local infoType, info = GetCursorInfo()
	
	if infoType == "battlepet" then
		svActivePet = info
		UpdatePetButtonIcon(false)
		ClearCursor()
		CheckForSummonPet()
	end
end)

--------------------------------------
--									--
--				TOOLTIP				--
--									--
--------------------------------------
petButton:SetScript("OnEnter", function(self, ...)
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
    GameTooltip:ClearLines()
	
	if svActivePet == nil then
		GameTooltip:AddLine("CLICK to open the pet journal, then")
		GameTooltip:AddLine("DRAG and DROP your pet here.")
	else
		local _,petCustomName,petLevel,petXP,_,_,_,petName = C_PetJournal.GetPetInfoByPetID(svActivePet)
		if petCustomName == nil then
			GameTooltip:AddLine(petName)
		else
			GameTooltip:AddLine(petCustomName)
		end
		GameTooltip:AddDoubleLine("Level:", petLevel, .8,.8,.8, 1,1,1)
		GameTooltip:AddDoubleLine("XP:", petXP, .8,.8,.8, 1,1,1)
		GameTooltip:AddDoubleLine("LEFT-CLICK:", "Summon Pet", .4,.4,1, .6,.6,1)
		GameTooltip:AddDoubleLine("RIGHT-CLICK:", "Clear Button", .4,.4,1, .6,.6,1)
		
		if svShowAlertText then
			GameTooltip:AddDoubleLine("SHIFT-RIGHT-CLICK:", "Alert Text Off", .4,.4,1, .6,.6,1)
		else
			GameTooltip:AddDoubleLine("SHIFT-RIGHT-CLICK:", "Alert Text On", .4,.4,1, .6,.6,1)
		end
		
		if svPlayAlertSound then
			GameTooltip:AddDoubleLine("CTRL-RIGHT-CLICK:", "Alert Sound Off", .4,.4,1, .6,.6,1)
		else
			GameTooltip:AddDoubleLine("CTRL-RIGHT-CLICK:", "Alert Sound On", .4,.4,1, .6,.6,1)
		end
	end

    GameTooltip:Show()
end)

petButton:SetScript("OnLeave", function(...)
	GameTooltip:Hide()
end)

--------------------------------------
--									--
--			PET HANDLING			--
--									--
--------------------------------------

-- Since we can't always rely on events to tell us when to summon/resummon, we're going to poll once
--	per second to see if we have the right pet out and that it's ok to summon.
function PetPollerInitialize()
	C_Timer.NewTicker(1, CheckForSummonPet)
end

-- Will unsummon an active pet if not in combat lockdown.
function UnsummonActivePet()
	if not InCombatLockdown() then
		local activePet = C_PetJournal.GetSummonedPetGUID()
		if activePet ~= nil then
			C_PetJournal.SummonPetByGUID(activePet)
		end
	end
end

-- This is where we try to autosummon or, if in combat, warn the player to click the button.
--	No checks are done here to validate the need to summon. This is only about what to summon
--	and how to do it.
function SummonActivePet()
	local activePet = C_PetJournal.GetSummonedPetGUID()
	if (activePet == nil) or (activePet ~= svActivePet) then
		if InCombatLockdown() then
			AlertSummon()
		else
			C_PetJournal.SummonPetByGUID(svActivePet)
		end
	end
end

function CheckForSummonPet()
	-- If we're not in a situation where we shouldn't summon, then check to see whether a summon is
	--	required.
	if UnitOnTaxi("PLAYER") then return end
	if UnitInVehicle("PLAYER") then return end
	if InCinematic() then return end
	if IsFlying() then return end
	
	-- If the player is stealthed and there's a pet out, unsummon the pet and bail out.
	if IsStealthed() then
		UnsummonActivePet()
		return
	end
	if svActivePet ~= nil then SummonActivePet() end
end

--------------------------------------
--									--
--	INITIALIZATION AND SAVED VARS	--
--									--
--------------------------------------

-- Saved variables aren't valid until ALL addons are loaded?
local loadTime
local varsNotLoaded = true

function LoadSavedVariables()
	if svShowAlertText == nil then svShowAlertText = true end
	if svPlayAlertSound == nil then svPlayAlertSound = true end
	if svButtonLoc == nil then
		petButton:SetPoint("CENTER",0,0)
		svButtonLoc = {}
		svButtonLoc.hpos = petButton:GetLeft()
		svButtonLoc.vpos = petButton:GetBottom()
	else
		petButton:SetPoint("BOTTOMLEFT", svButtonLoc.hpos, svButtonLoc.vpos)
	end
	UpdatePetButtonIcon()
	petButton:Show()
	varsNotLoaded = false
end

-- Checks to see if we've waited long enough for our saved variables to be reliably readable.
--	If we have, then vars are loaded and other initializations are performed.
function CheckOkToLoad()
	if loadTime < GetTime() and varsNotLoaded then
		LoadSavedVariables()
		AlertFrameInitialize()	-- Alert frame used to tell the player to summon the active pet.
		PetPollerInitialize()	-- Periodic check to see if pet needs to be resummoned.
		CheckForSummonPet()		-- Make sure our pet is out.
		print(swmNameFormatted .. ": Loaded.")
	end
end

--------------------------------------
--									--
--	TIMING AND EVENT HANDLING		--
--									--
--------------------------------------

petButton:SetScript("OnEvent", function(self, event, ...)
	-- Every time we get hit with an ADDON_LOADED event, regardless of which addon, we need to
	--	give ourselves time for things to settle. If we don't, some really weighty addons can prevent
	--	our saved vars from being loaded right away, even if we've received a loaded event for this
	--	addon. So the strategy here is to delay loading our variables for a couple of seconds for
	--	every addon load we see. This seems to produce a very stable behavior regardless of how many
	--	addons the user has or how big they are.
	if event == "ADDON_LOADED" and varsNotLoaded then
		loadTime = GetTime() + 2
		C_Timer.After(3, CheckOkToLoad)
	elseif event == "PLAYER_ENTERING_WORLD" then
		CheckForSummonPet()
	elseif event == "PLAYER_REGEN_DISABLED" then
		CheckForSummonPet()
	end
end)
